/**
 * JimBot - Java IM Bot
 * Copyright (C) 2006-2009 JimBot project
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package ru.jimbot.protocol;
import ru.caffeineim.protocols.icq.core.OscarConnection;
import ru.caffeineim.protocols.icq.exceptions.ConvertStringException;
import ru.caffeineim.protocols.icq.integration.OscarInterface;
import ru.caffeineim.protocols.icq.integration.events.*;
import ru.caffeineim.protocols.icq.integration.listeners.MessagingListener;
import ru.caffeineim.protocols.icq.integration.listeners.OurStatusListener;
import ru.caffeineim.protocols.icq.integration.listeners.XStatusListener;
import ru.caffeineim.protocols.icq.setting.enumerations.ClientModeEnum;
import ru.caffeineim.protocols.icq.setting.enumerations.StatusModeEnum;
import ru.caffeineim.protocols.icq.setting.enumerations.XStatusModeEnum;
import ru.jimbot.modules.AbstractProps;
import ru.jimbot.modules.MsgOutQueue;
import ru.jimbot.util.Log;
import ru.jimbot.util.MainProps;

/**
 * @author Prolubnikov Dmitry, fraer72
 */
public class IcqProtocol extends AbstractProtocol
implements OurStatusListener,
           MessagingListener,
           XStatusListener{
private OscarConnection con = null;
private AbstractProps props;
int xStatusId;
String xStatusText;
private boolean connected = false;

public IcqProtocol(AbstractProps props){
this.props = props;
server = MainProps.getServer();
port = MainProps.getPort();
mq = new MsgOutQueue(this, props.getIntProperty("bot.pauseOut"),
props.getIntProperty("bot.pauseRestart"),
props.getIntProperty("bot.msgOutLimit"));
}

public AbstractProps getProps(){
return props;
}

public int getOuteqSize(){
return mq.size();
}

public void connect() {
Log.info("Connect uin - " + screenName);
mq.start();
con = new OscarConnection(server, port, screenName, password, props.getBooleanProperty("web.aware.on.off"), false);
con.addOurStatusListener(this);
con.addMessagingListener(this);
con.addXStatusListener(this);

con.connect();

         // TODO:     onAuthorizationFailed(), onLogin(),   
//connected = true;
}

public void reConnect(){
try {
con.close();
con.removeOurStatusListener(this);
con.removeMessagingListener(this);
con.removeXStatusListener(this);
con = null;
}catch (Exception e){
e.printStackTrace();
}
con = new OscarConnection(server, port, screenName, password, props.getBooleanProperty("web.aware.on.off"), false);
con.addOurStatusListener(this);
con.addMessagingListener(this);
con.addXStatusListener(this);
con.connect();
}

public void disconnect() {
if(con == null) return;
try {
mq.stop();
con.close();
con.removeOurStatusListener(this);
con.removeMessagingListener(this);
con.removeXStatusListener(this);
con = null;
}catch (Exception ex){
ex.printStackTrace();
}
}

public void getMsg(String sendSN, String recivSN, String msg,
boolean isOffline){
protList.getMsg(sendSN, recivSN, msg, isOffline);
}


public boolean isOnLine() {
if(con == null) return false;
return connected;
}

public void sendMsg(String sn, String msg){
try{
OscarInterface.sendBasicMessage(con, sn, msg);
}catch (ConvertStringException e){
Log.info("ERROR send message: " + msg);
e.printStackTrace();
}
}


public boolean isNoAuthUin(String uin){
return props.getBooleanProperty("chat.isAuthRequest");
}

public void authRequest(String uin, String msg){}


public void onIncomingMessage(IncomingMessageEvent e){
if(MainProps.isIgnor(e.getSenderID())){
Log.flood2("IGNORE LIST: " + e.getMessageId() + "->" + screenName + ": " + e.getMessage());
return;
}

 if(e.getMessage().equals("")||e.getMessage()==null) {
    //  MainProps.setBadlist(e.getSenderID(), e.getMessage());
        return;
     }
if(e.getSenderID().equals("1")){
Log.error("   ICQ.    ...");
try{
connected = false;
}catch (Exception ex) {
ex.printStackTrace();
}
return;
}
protList.getMsg(e.getSenderID(), screenName, e.getMessage(), false);
}

/**
 UNKNOWN_ERROR           = 0; "Unknown Error"
 BAD_UIN_ERROR           = 1; "Bad UIN.";
 PASSWORD_ERROR          = 2; "Password incorrect.";
 NOT_EXISTS_ERROR        = 3; "This ICQ number does not exist.";
 LIMIT_EXCEEDED_ERROR    = 4; "Rate limit exceeded. Please try to reconnect in a few minutes."
 MAXIMUM_USERS_IP_ERROR  = 5; "The amount of users connected from this IP has reached the maximum."
 OLDER_ICQ_VERSION_ERROR = 6; "You are using an older version of ICQ. Please upgrade."
 CANT_REGISTER_ERROR     = 7; "Can't register on the ICQ network. Reconnect in a few minutes."
 */

public void onAuthorizationFailed(LoginErrorEvent e){
Log.error(" uin` " + screenName +  "    ICQ  . : " +  e.getErrorMessage());
connected = false;
}

public void onStatusChange(StatusEvent e){
Log.debug("StatusEvent: " + e.getStatusMode());
}

public void onXStatusRequest(XStatusRequestEvent e) {
try {
 int xstatus=Integer.parseInt(props.getStringProperty("icq.xstatus1"));
OscarInterface.sendXStatus(con, new XStatusModeEnum(xstatus),
props.getStringProperty("icq.STATUS_MESSAGE1"),
props.getStringProperty("icq.STATUS_MESSAGE2"), e.getTime(), e.getMsgID(), e.getSenderID(), e.getSenderTcpVersion());
}catch(ConvertStringException ex) {
System.err.println(ex.getMessage());
}
}

public void onLogin() {
connected = true;
int status=Integer.parseInt(props.getStringProperty("icq.status1"));
 int xstatus=Integer.parseInt(props.getStringProperty("icq.xstatus1"));
 int client=Integer.parseInt(props.getStringProperty("icq.client1"));
OscarInterface.changeStatus(con, new StatusModeEnum(status, props.getBooleanProperty("ball.of.joy.on.off")));
OscarInterface.changeXStatus(con, new XStatusModeEnum(xstatus), new ClientModeEnum(client));
Log.talk("UIN - " + screenName + " online");
}

public void onLogout(Exception e) {
Log.error(" : " + screenName + " - " + server + ":" + port +
"  : " + e.getMessage());
connected = false;
}

/**
 * @param n -  -
 *
 * @param text -  -
 */

public void setXStatus( int n, String text ){
if ( n >= 0 && n <= 37 ){
xStatusId = n;
props.setStringProperty( "icq.xstatus1", Integer.toString(xStatusId) );
}
if ( !text.equals( "" ) ){
xStatusText = text;
props.setStringProperty("icq.STATUS_MESSAGE2", xStatusText);
}
 int client=Integer.parseInt(props.getStringProperty("icq.client1"));
OscarInterface.changeXStatus(con, new XStatusModeEnum(xStatusId), new ClientModeEnum(client));
}

public void setaStatus() {
//connected = true;
int status=Integer.parseInt(props.getStringProperty("icq.status1"));
OscarInterface.changeStatus(con, new StatusModeEnum(status, props.getBooleanProperty("ball.of.joy.on.off")));

}

/**
 *  - 
 * @param number
 */

public void setXStatusNumber( int number ){
     int client=Integer.parseInt(props.getStringProperty("icq.client1"));
OscarInterface.changeXStatus(con, new XStatusModeEnum(number), new ClientModeEnum(client));
}

    public void onMessageMissed(MessageMissedEvent e) {
    Log.debug("Message from " + e.getUin() + " can't be recieved because " + e.getReason()  +
                                " count="+e.getMissedMsgCount());
    }

    public void onMessageError(MessageErrorEvent e) {
    Log.error("Message error " + e.getError().toString());
    }

    public void onStatusResponse(StatusEvent se) {}
    public void setStatus(int status) {}
    public void getStatus(String sn, int status) {}
    public void addContactList(String sn) {}
    public void RemoveContactList(String sn) {}
    public void onIncomingUrl(IncomingUrlEvent iue) {}
    public void onOfflineMessage(OfflineMessageEvent ome) {}
    public void onMessageAck(MessageAckEvent mae) {}
    public void onXStatusResponse(XStatusResponseEvent xsre) {}



}
